unit UMainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin, Buttons;

type
  TMainForm = class(TForm)
    gbResearch2: TGroupBox;
    lstResearchType: TListBox;
    lblResearchType: TLabel;
    seResearchLevel2: TSpinEdit;
    lblResearchLevel2: TLabel;
    lblResearchTurns: TLabel;
    lblTurnsValue: TLabel;
    gbResearch1: TGroupBox;
    lblOrgValue: TLabel;
    lblResearchLevel: TLabel;
    lblIncrementByResearch: TLabel;
    lblIncrementByResearchValue: TLabel;
    lblNewResearchedValue: TLabel;
    lblNewResValue: TLabel;
    edOrgValue: TSpinEdit;
    edResearchLevel: TSpinEdit;
    lblTo: TLabel;
    lblTargetResearch: TLabel;
    gbResearch3: TGroupBox;
    lblCostParam: TLabel;
    lblByCostValue: TLabel;
    lblNewCostValue: TLabel;
    lblMatPerTurn: TLabel;
    edCost: TSpinEdit;
    edMatPerTurn: TSpinEdit;
    lblResearch3: TLabel;
    seResearch3: TSpinEdit;
    lblNewCost: TLabel;
    lblIncByResearch: TLabel;
    gbUpgrades: TGroupBox;
    seBaseValue: TSpinEdit;
    lblBaseValue: TLabel;
    lblCostType: TLabel;
    lstCostType: TListBox;
    btnValueDec: TSpeedButton;
    btnValueInc: TSpeedButton;
    lblUpgradeValue: TLabel;
    lblGoldSum: TLabel;
    lblGoldValueSum: TLabel;
    lblGoldNext: TLabel;
    lblGoldValue: TLabel;
    lblNewUpgValue: TLabel;
    procedure edOrgValueChange(Sender: TObject);
    procedure edResearchLevelChange(Sender: TObject);
    procedure edCostChange(Sender: TObject);
    procedure edMatPerTurnChange(Sender: TObject);
    procedure seResearchLevel2Change(Sender: TObject);
    procedure lstResearchTypeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure seResearch3Change(Sender: TObject);
    procedure btnValueDecClick(Sender: TObject);
    procedure btnValueIncClick(Sender: TObject);
    procedure seBaseValueChange(Sender: TObject);
    procedure lstCostTypeClick(Sender: TObject);
    procedure seMatPerTurnChange(Sender: TObject);
  private
    upgradedValue: integer;
    procedure CalculateResearch();
    procedure CalculateResearch2();
    procedure CalculateResearch3();
    procedure CalculateUpgrade();
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  UResearchUpgrade;

procedure TMainForm.edOrgValueChange(Sender: TObject);
begin
  CalculateResearch2();
end;

procedure TMainForm.edResearchLevelChange(Sender: TObject);
begin
  CalculateResearch2();
end;

procedure TMainForm.edCostChange(Sender: TObject);
begin
  CalculateResearch2();
end;

procedure TMainForm.edMatPerTurnChange(Sender: TObject);
begin
  if Sender <> nil then begin
    edCost.Increment := edMatPerTurn.Value;
    edCost.MinValue := edMatPerTurn.Value;
  end;  
  CalculateResearch3();
end;

procedure TMainForm.seResearchLevel2Change(Sender: TObject);
begin
  CalculateResearch();
end;

procedure TMainForm.lstResearchTypeClick(Sender: TObject);
begin
  CalculateResearch();
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  lstResearchType.ItemIndex := 0;
  lstCostType.ItemIndex := 0;
  upgradedValue := seBaseValue.Value;
  CalculateResearch();
  CalculateResearch2();
  CalculateResearch3();
  CalculateUpgrade();
end;

procedure TMainForm.seResearch3Change(Sender: TObject);
begin
  CalculateResearch3();
end;

procedure TMainForm.btnValueDecClick(Sender: TObject);
begin
  if upgradedValue > seBaseValue.Value then
    upgradedValue := upgradedValue - CalcIncreaseByUpgrade(seBaseValue.Value);
  CalculateUpgrade();
end;

procedure TMainForm.btnValueIncClick(Sender: TObject);
begin
  upgradedValue := upgradedValue + CalcIncreaseByUpgrade(seBaseValue.Value);
  CalculateUpgrade();
end;

procedure TMainForm.seBaseValueChange(Sender: TObject);
begin
  upgradedValue := seBaseValue.Value;
  CalculateUpgrade();
end;

procedure TMainForm.lstCostTypeClick(Sender: TObject);
begin
  CalculateUpgrade();
end;

procedure TMainForm.seMatPerTurnChange(Sender: TObject);
begin
  CalculateUpgrade();
end;

procedure TMainForm.CalculateResearch;
var
  resVal, turns: integer;
  uType: TUpgradeTypes;
begin
  if lstResearchType.ItemIndex < 0 then
    Exit;
  try
    resVal := seResearchLevel2.Value;
    uType := TUpgradeTypes(lstResearchType.ItemIndex);

    turns := CalcResearchTurns(resVal, uType);
    lblTurnsValue.Caption := IntToStr(turns) + ' turns';
    lblTargetResearch.Caption :=
      IntToStr(100 + resVal + CalcIncreaseByResearch()) + '%';
  except
  end;
end;

procedure TMainForm.CalculateResearch2;
var
  baseVal, resVal, dVal: integer;
begin
  try
    resVal := edResearchLevel.Value;

    baseVal := edOrgValue.Value;
    dVal := CalcChangeByResearch(baseVal, resVal);
    lblIncrementByResearchValue.Caption := IntToStr(dVal);
    lblNewResValue.Caption := IntToStr(baseVal + dVal);
  except
  end;
end;

procedure TMainForm.CalculateResearch3;
var
  resVal: integer;
  orgCost, matPerTurn, dCostVal: integer;
begin
  try
    resVal := seResearch3.Value;

    orgCost := edCost.Value;
    matPerTurn := edMatPerTurn.Value;
    dCostVal := CalcChangeByResearch(orgCost, resVal, utCost, matPerTurn);
    lblByCostValue.Caption := IntToStr(dCostVal);
    lblNewCostValue.Caption := IntToStr(orgCost + dCostVal);
  except
  end;
end;

procedure TMainForm.CalculateUpgrade();
var
  baseVal, curVal, resVal: integer;
  uType: TUpgradeTypes;
  gold, goldSum: integer;
begin
  try
    baseVal := seBaseValue.Value;
    curVal := baseVal; // For test. In real can be different.
    resVal := 0; // No current research by Lab.
    uType := TUpgradeTypes(lstCostType.ItemIndex);

    lblUpgradeValue.Caption :=
      IntToStr(upgradedValue) + ' (+' + IntToStr(upgradedValue - baseVal) + ')';
    gold := CalcPrice(upgradedValue, baseVal, uType, resVal);
    goldSum := GetCostForUpgrade(baseVal, curVal, upgradedValue, resVal, uType);

    if goldSum <> NO_PRICE_AVAILABLE then
      lblGoldValueSum.Caption := IntToStr(goldSum)
    else
      lblGoldValueSum.Caption := 'Wrong new value';
    if gold <> NO_PRICE_AVAILABLE then
      lblGoldValue.Caption := IntToStr(gold)
    else
      lblGoldValue.Caption := 'Can not upgrade';
    btnValueInc.Enabled := gold <> NO_PRICE_AVAILABLE;
    btnValueDec.Enabled := upgradedValue > seBaseValue.Value;
  except
  end;
end;

end.
